@@ -73,10 +73,13 @@ static GtkTargetEntry drop_types [] = {
 	{ NAUTILUS_ICON_DND_COLOR_TYPE, 0, NAUTILUS_ICON_DND_COLOR },
 	{ NAUTILUS_ICON_DND_BGIMAGE_TYPE, 0, NAUTILUS_ICON_DND_BGIMAGE },
 	{ NAUTILUS_ICON_DND_KEYWORD_TYPE, 0, NAUTILUS_ICON_DND_KEYWORD },
-	{ NAUTILUS_ICON_DND_RESET_BACKGROUND_TYPE,  0, NAUTILUS_ICON_DND_RESET_BACKGROUND }
+	{ NAUTILUS_ICON_DND_RESET_BACKGROUND_TYPE,  0, NAUTILUS_ICON_DND_RESET_BACKGROUND },
+	/* Must be last: */
+	{ NAUTILUS_ICON_DND_ROOTWINDOW_DROP_TYPE,  0, NAUTILUS_ICON_DND_ROOTWINDOW_DROP }
 };
 
 static GtkTargetList *drop_types_list = NULL;
+static GtkTargetList *drop_types_list_root = NULL;
 
 static EelCanvasItem *
 create_selection_shadow (NautilusIconContainer *container,
@@ -98,7 +101,7 @@ create_selection_shadow (NautilusIconContainer *container,
 		return NULL;
 	}
 		
-	stipple = container->details->dnd_info->drag_info.stipple;
+	stipple = container->details->dnd_info->stipple;
 	g_return_val_if_fail (stipple != NULL, NULL);
 
 	canvas = EEL_CANVAS (container);
@@ -366,21 +369,43 @@ static void
 get_data_on_first_target_we_support (GtkWidget *widget, GdkDragContext *context, guint32 time)
 {
 	GList *target;
-
+	GtkTargetList *list;
+	
 	if (drop_types_list == NULL)
 		drop_types_list = gtk_target_list_new (drop_types,
-						       G_N_ELEMENTS (drop_types));
+						       G_N_ELEMENTS (drop_types) - 1);
+	if (drop_types_list_root == NULL)
+		drop_types_list_root = gtk_target_list_new (drop_types,
+							    G_N_ELEMENTS (drop_types));
 
+	if (nautilus_icon_container_get_is_desktop (NAUTILUS_ICON_CONTAINER (widget))) {
+		list = drop_types_list_root;
+	} else {
+		list = drop_types_list;
+	}
+	
 	for (target = context->targets; target != NULL; target = target->next) {
-		guint dummy_info;
+		guint info;
 		GdkAtom target_atom = GDK_POINTER_TO_ATOM (target->data);
+		NautilusDragInfo *drag_info;
 
-		if (gtk_target_list_find (drop_types_list, 
+		drag_info = &(NAUTILUS_ICON_CONTAINER (widget)->details->dnd_info->drag_info);
+
+		if (gtk_target_list_find (list, 
 					  target_atom,
-					  &dummy_info)) {
-			gtk_drag_get_data (GTK_WIDGET (widget), context,
-					   target_atom,
-					   time);
+					  &info)) {
+			/* Don't get_data for rootwindow drops unless it's the actual drop */
+			if (info == NAUTILUS_ICON_DND_ROOTWINDOW_DROP &&
+			    !drag_info->drop_occured) {
+				/* We can't call get_data here, because that would
+				   make the source execute the rootwin action */
+				drag_info->got_drop_data_type = TRUE;
+				drag_info->data_type = NAUTILUS_ICON_DND_ROOTWINDOW_DROP;
+			} else {
+				gtk_drag_get_data (GTK_WIDGET (widget), context,
+						   target_atom,
+						   time);
+			}
 			break;
 		}
 	}
@@ -495,13 +520,56 @@ nautilus_icon_container_selection_items_local (NautilusIconContainer *container,
 	return result;
 }
 
+static GdkDragAction 
+get_background_drag_action (NautilusIconContainer *container, 
+			    GdkDragAction action)
+{
+	/* FIXME: This function is very FMDirectoryView specific, and
+	 * should be moved out of nautilus-icon-dnd.c */
+	GdkDragAction valid_actions;
+
+	if (action == GDK_ACTION_ASK) {
+		valid_actions = NAUTILUS_DND_ACTION_SET_AS_BACKGROUND;
+		if (g_object_get_data (G_OBJECT (eel_get_widget_background (GTK_WIDGET (container))), "is_desktop") == 0) {
+			valid_actions |= NAUTILUS_DND_ACTION_SET_AS_GLOBAL_BACKGROUND;
+		}
+
+		action = nautilus_drag_drop_background_ask (valid_actions);
+	}
+
+	return action;
+}
+
+static void
+receive_dropped_color (NautilusIconContainer *container,
+		       int x, int y,
+		       GdkDragAction action,
+		       GtkSelectionData *data)
+{
+	action = get_background_drag_action (container, action);
+	
+	if (action > 0) {
+		eel_background_receive_dropped_color
+			(eel_get_widget_background (GTK_WIDGET (container)),
+			 GTK_WIDGET (container), 
+			 action, x, y, data);
+	}
+}
+
 /* handle dropped tile images */
 static void
-receive_dropped_tile_image (NautilusIconContainer *container, GtkSelectionData *data)
+receive_dropped_tile_image (NautilusIconContainer *container, GdkDragAction action, GtkSelectionData *data)
 {
 	g_assert (data != NULL);
-	eel_background_receive_dropped_background_image
-		(eel_get_widget_background (GTK_WIDGET (container)), data->data);
+
+	action = get_background_drag_action (container, action);
+
+	if (action > 0) {
+		eel_background_receive_dropped_background_image
+			(eel_get_widget_background (GTK_WIDGET (container)), 
+			 action, 
+			 data->data);
+	}
 }
 
 /* handle dropped keywords */
@@ -890,6 +958,7 @@ nautilus_icon_container_receive_dropped_icons (NautilusIconContainer *container,
 		selected_item = container->details->dnd_info->drag_info.selection_list->data;
 		eel_background_receive_dropped_background_image
 			(eel_get_widget_background (GTK_WIDGET (container)),
+			 context->action,
 			 selected_item->uri);
 		return;
 	}
@@ -934,7 +1003,7 @@ nautilus_icon_container_get_drop_action (NautilusIconContainer *container,
 	gboolean icon_hit;
 	NautilusIcon *icon;
 	double world_x, world_y;
-	
+
 	icon_hit = FALSE;
 	if (!container->details->dnd_info->drag_info.got_drop_data_type) {
 		/* drag_data_received_callback didn't get called yet */
@@ -983,6 +1052,7 @@ nautilus_icon_container_get_drop_action (NautilusIconContainer *container,
 	
 	case NAUTILUS_ICON_DND_URI_LIST:
 	case NAUTILUS_ICON_DND_URL:
+	case NAUTILUS_ICON_DND_ROOTWINDOW_DROP:
 		*action = context->suggested_action;
 		break;
 
@@ -1297,7 +1367,7 @@ drag_data_received_callback (GtkWidget *widget,
     	NautilusDragInfo *drag_info;
 	EelBackground *background;
 	gboolean success;
-	
+
 	drag_info = &(NAUTILUS_ICON_CONTAINER (widget)->details->dnd_info->drag_info);
 
 	drag_info->got_drop_data_type = TRUE;
@@ -1326,9 +1396,11 @@ drag_data_received_callback (GtkWidget *widget,
 			drag_info->selection_data = gtk_selection_data_copy (data);
 		}
 		break;
+	case NAUTILUS_ICON_DND_ROOTWINDOW_DROP:
+		/* Do nothing, this won't even happen, since we don't want to call get_data twice */
+		break;
 	}
 
-
 	/* this is the second use case of this callback.
 	 * we have to do the actual work for the drop.
 	 */
@@ -1342,14 +1414,16 @@ drag_data_received_callback (GtkWidget *widget,
 				 context, x, y);
 			break;
 		case NAUTILUS_ICON_DND_COLOR:
-			eel_background_receive_dropped_color
-				(eel_get_widget_background (widget),
-				 widget, x, y, data);
+			receive_dropped_color (NAUTILUS_ICON_CONTAINER (widget),
+					       x, y,
+					       context->action,
+					       data);
 			success = TRUE;
 			break;
 		case NAUTILUS_ICON_DND_BGIMAGE:
 			receive_dropped_tile_image
 				(NAUTILUS_ICON_CONTAINER (widget),
+				 context->action,
 				 data);
 			break;
 		case NAUTILUS_ICON_DND_KEYWORD:
@@ -1371,6 +1445,9 @@ drag_data_received_callback (GtkWidget *widget,
 			}
 			gtk_drag_finish (context, FALSE, FALSE, time);			
 			break;
+		case NAUTILUS_ICON_DND_ROOTWINDOW_DROP:
+			/* Do nothing, everything is done by the sender */
+			break;
 		}
 		gtk_drag_finish (context, success, FALSE, time);
 		
@@ -1384,25 +1461,47 @@ drag_data_received_callback (GtkWidget *widget,
 
 }
 
+void
+nautilus_icon_dnd_set_stipple (NautilusIconContainer *container,
+			       GdkBitmap             *stipple)
+{
+	if (stipple != NULL) {
+		g_object_ref (stipple);
+	}
+	
+	if (container->details->dnd_info->stipple != NULL) {
+		g_object_unref (container->details->dnd_info->stipple);
+	}
+
+	container->details->dnd_info->stipple = stipple;
+}
+
 void
 nautilus_icon_dnd_init (NautilusIconContainer *container,
 			GdkBitmap *stipple)
 {
+	int n_elements;
+	
 	g_return_if_fail (container != NULL);
 	g_return_if_fail (NAUTILUS_IS_ICON_CONTAINER (container));
 
 
 	container->details->dnd_info = g_new0 (NautilusIconDndInfo, 1);
 	nautilus_drag_init (&container->details->dnd_info->drag_info,
-		drag_types, G_N_ELEMENTS (drag_types), stipple);
+		drag_types, G_N_ELEMENTS (drag_types));
 
 	/* Set up the widget as a drag destination.
 	 * (But not a source, as drags starting from this widget will be
          * implemented by dealing with events manually.)
 	 */
+	n_elements = G_N_ELEMENTS (drop_types);
+	if (!nautilus_icon_container_get_is_desktop (container)) {
+		/* Don't set up rootwindow drop */
+		n_elements -= 1;
+	}
 	gtk_drag_dest_set (GTK_WIDGET (container),
 			   0,
-			   drop_types, G_N_ELEMENTS (drop_types),
+			   drop_types, n_elements,
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
 
 	/* Messages for outgoing drag. */
@@ -1421,6 +1520,9 @@ nautilus_icon_dnd_init (NautilusIconContainer *container,
 	g_signal_connect (container, "drag_leave",
 			  G_CALLBACK (drag_leave_callback), NULL);
 
+	if (stipple != NULL) {
+		container->details->dnd_info->stipple = g_object_ref (stipple);
+	}
 }
 
 void
@@ -1431,6 +1533,10 @@ nautilus_icon_dnd_fini (NautilusIconContainer *container)
 	if (container->details->dnd_info != NULL) {
 		stop_auto_scroll (container);
 
+		if (container->details->dnd_info->stipple != NULL) {
+			g_object_unref (container->details->dnd_info->stipple);
+		}
+
 		nautilus_drag_finalize (&container->details->dnd_info->drag_info);
 		container->details->dnd_info = NULL;
 	}
