@@ -464,25 +464,28 @@ nautilus_window_zoom_to_fit (NautilusWindow *window)
  * (The user can still stretch the window taller if desired).
  */
 static guint
-get_max_forced_height (void)
+get_max_forced_height (GdkScreen *screen)
 {
-	return (gdk_screen_height () * 90) / 100;
+	return (gdk_screen_get_height (screen) * 90) / 100;
 }
 
 /* Code should never force the window wider than this size.
  * (The user can still stretch the window wider if desired).
  */
 static guint
-get_max_forced_width (void)
+get_max_forced_width (GdkScreen *screen)
 {
-	return (gdk_screen_width () * 90) / 100;
+	return (gdk_screen_get_width (screen) * 90) / 100;
 }
 
 static void
 set_initial_window_geometry (NautilusWindow *window)
 {
+	GdkScreen *screen;
 	guint max_width_for_screen, max_height_for_screen;
 
+	screen = gtk_window_get_screen (GTK_WINDOW (window));
+	
 	/* Don't let GTK determine the minimum size
 	 * automatically. It will insist that the window be
 	 * really wide based on some misguided notion about
@@ -498,9 +501,9 @@ set_initial_window_geometry (NautilusWindow *window)
 	 * probably remove this broken set_size_request() here.
 	 * - hp@redhat.com
 	 */
-	
-	max_width_for_screen = get_max_forced_width ();
-	max_height_for_screen = get_max_forced_height ();
+
+	max_width_for_screen = get_max_forced_width (screen);
+	max_height_for_screen = get_max_forced_height (screen);
 
 	gtk_widget_set_size_request (GTK_WIDGET (window), 
 				     MIN (NAUTILUS_WINDOW_MIN_WIDTH, 
@@ -1025,6 +1028,7 @@ static void
 nautilus_window_size_request (GtkWidget		*widget,
 			      GtkRequisition	*requisition)
 {
+	GdkScreen *screen;
 	guint max_width;
 	guint max_height;
 
@@ -1033,6 +1037,8 @@ nautilus_window_size_request (GtkWidget		*widget,
 
 	GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
 
+	screen = gtk_window_get_screen (GTK_WINDOW (widget));
+
 	/* Limit the requisition to be within 90% of the available screen 
 	 * real state.
 	 *
@@ -1046,8 +1052,8 @@ nautilus_window_size_request (GtkWidget		*widget,
 	 * to be requested, should still be fixed.  This code is here only to 
 	 * prevent the extremely frustrating consequence of such bugs.
 	 */
-	max_width = get_max_forced_width ();
-	max_height = get_max_forced_height ();
+	max_width = get_max_forced_width (screen);
+	max_height = get_max_forced_height (screen);
 
 	if (requisition->width > (int) max_width) {
 		requisition->width = max_width;
@@ -2011,12 +2017,14 @@ nautilus_window_hide_location_bar (NautilusWindow *window)
 {
 	window->details->temporary_navigation_bar = FALSE;
 	hide_dock_item (window, LOCATION_BAR_PATH);
+	eel_preferences_set_boolean (NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR, FALSE);
 }
 
 void 
 nautilus_window_show_location_bar (NautilusWindow *window)
 {
 	show_dock_item (window, LOCATION_BAR_PATH);
+	eel_preferences_set_boolean (NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR, TRUE);
 }
 
 gboolean
@@ -2029,6 +2037,7 @@ void
 nautilus_window_hide_toolbar (NautilusWindow *window)
 {
 	hide_dock_item (window, TOOLBAR_PATH);
+	eel_preferences_set_boolean (NAUTILUS_PREFERENCES_START_WITH_TOOLBAR, FALSE);
 }
 
 void 
@@ -2036,6 +2045,7 @@ nautilus_window_show_toolbar (NautilusWindow *window)
 {
 	nautilus_window_activate_throbber (window);
 	show_dock_item (window, TOOLBAR_PATH);
+	eel_preferences_set_boolean (NAUTILUS_PREFERENCES_START_WITH_TOOLBAR, TRUE);
 }
 
 gboolean
@@ -2053,6 +2063,7 @@ nautilus_window_hide_sidebar (NautilusWindow *window)
 
 	nautilus_window_tear_down_sidebar (window);
 	nautilus_window_update_show_hide_menu_items (window);
+	eel_preferences_set_boolean (NAUTILUS_PREFERENCES_START_WITH_SIDEBAR, FALSE);
 }
 
 void 
@@ -2064,6 +2075,7 @@ nautilus_window_show_sidebar (NautilusWindow *window)
 
 	nautilus_window_set_up_sidebar (window);
 	nautilus_window_update_show_hide_menu_items (window);
+	eel_preferences_set_boolean (NAUTILUS_PREFERENCES_START_WITH_SIDEBAR, TRUE);
 }
 
 gboolean
@@ -2081,6 +2093,7 @@ nautilus_window_hide_status_bar (NautilusWindow *window)
 	hide_dock_item (window, STATUS_BAR_PATH);
 
 	nautilus_window_update_show_hide_menu_items (window);
+	eel_preferences_set_boolean (NAUTILUS_PREFERENCES_START_WITH_STATUS_BAR, FALSE);
 }
 
 void 
@@ -2089,6 +2102,7 @@ nautilus_window_show_status_bar (NautilusWindow *window)
 	show_dock_item (window, STATUS_BAR_PATH);
 
 	nautilus_window_update_show_hide_menu_items (window);
+	eel_preferences_set_boolean (NAUTILUS_PREFERENCES_START_WITH_STATUS_BAR, TRUE);
 }
 
 gboolean
